@@ -72,9 +72,9 @@ DEFINE_TEST(test_write_open_memory)
 			    archive_write_close(a));
 		else {
 			assertEqualIntA(a, ARCHIVE_OK, archive_write_close(a));
-			assertEqualInt(used, archive_position_compressed(a));
-			assertEqualInt(archive_position_compressed(a),
-			    archive_position_uncompressed(a));
+			assertEqualInt(used, archive_filter_bytes(a, -1));
+			assertEqualInt(archive_filter_bytes(a, -1),
+			    archive_filter_bytes(a, 0));
 		}
 		assertEqualInt(ARCHIVE_OK, archive_write_free(a));
 		assertEqualInt(buff[i], 0xAE);
