@@ -17,10 +17,12 @@ DEFINE_TEST(test_help)
 DEFINE_TEST(test_leading_slash)
 DEFINE_TEST(test_missing_file)
 DEFINE_TEST(test_option_a)
+DEFINE_TEST(test_option_acls)
 DEFINE_TEST(test_option_b64encode)
 DEFINE_TEST(test_option_b)
 DEFINE_TEST(test_option_C_upper)
 DEFINE_TEST(test_option_exclude)
+DEFINE_TEST(test_option_fflags)
 DEFINE_TEST(test_option_gid_gname)
 DEFINE_TEST(test_option_grzip)
 DEFINE_TEST(test_option_H_upper)
@@ -45,6 +47,7 @@ DEFINE_TEST(test_option_T_upper)
 DEFINE_TEST(test_option_uid_uname)
 DEFINE_TEST(test_option_uuencode)
 DEFINE_TEST(test_option_U_upper)
+DEFINE_TEST(test_option_xattrs)
 DEFINE_TEST(test_option_X_upper)
 DEFINE_TEST(test_option_xz)
 DEFINE_TEST(test_option_z)
